/**
   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if a string cannot be parsed.

   The static method parse() is a recursive descent parser for JSON strings.

   Here is a EBNF grammar for JSON strings.

 object -> '{' [ pair ( ',' pair)* ] '}'

   pair -> string ':' value

  array -> '[' [ value ( ',' value)* ] ']'

  value -> object   // begins with '{'
         | array    // begins with '['
         | true
         | false
         | null
         | string
         | number
*/

public class ParseJSON_No_Error_Checking
{
   public static void parse(String prefix)
   {
      Tokenizer tokens = new Tokenizer(prefix);

      getObject(tokens);        // parse the token stream
   }//parse()






   private static void getObject(Tokenizer tokens)
   {
      tokens.match("{");

      String token = tokens.peekToken();  // one token look ahead

      if ( ! token.equals("}") )
      {
         getPair(tokens);

         token = tokens.peekToken();  // one token look ahead

         while ( token.equals(",") )
         {
            tokens.nextToken();  // consume the comma
            getPair(tokens);
            token = tokens.peekToken();  // one token look ahead
         }
      }

      tokens.match("}");
    }//getObject()


   private static void getPair(Tokenizer tokens)
   {
      tokens.nextToken();  // consume the string

      tokens.match(":");

      getValue(tokens);

   }//getPair()


   private static void getArray(Tokenizer tokens)
   {
      tokens.match("[");

      String token = tokens.peekToken();  // one token look ahead

      if ( ! token.equals("]") )
      {
         getValue(tokens);

         token = tokens.peekToken();  // one token look ahead

         while ( token.equals(",") )
         {
            tokens.nextToken();  // consume the comma
            getValue(tokens);
            token = tokens.peekToken();  // one token look ahead
         }
      }

      tokens.match("]");
   }//getArray()


   private static void getValue(Tokenizer tokens)
   {
      String token = tokens.peekToken();  // one token look ahead

      if ( token.equals("{") )
      {
         getObject(tokens);
      }
      else if ( token.equals("[") )
      {
         getArray(tokens);
      }
      else if ( token.equals("true") )
      {
         tokens.nextToken();    // consume the literal
      }
      else if ( token.equals("false") )
      {
         tokens.nextToken();     // consume the literal
      }
      else if ( token.equals("null") )
      {
         tokens.nextToken();     // consume the literal
      }
      else  // number or string
      {
         tokens.nextToken();
      }

   }//getValue()

}//ParseJSON_No_Error_Checking