/*
   Silly Language S1.

   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if the string cannot be parsed.

   Notice that these productions are mutually recursive, but no production is recursive.

   S1 -> A '+' B
       | B '+' A

   A  -> 'a' '*' '(' S1 ')'
       | 'a'

   B  -> 'b' '*' '(' S1 ')'
       | 'b'

   We can use EBNF to rewrite this grammar.

   S1 -> A '+' B
       | B '+' A

   A  -> 'a' [ '*' '(' S1 ')' ]

   B  -> 'b' [ '*' '(' S1 ')' ]

   Some example strings are:
   "a*(a+b)+b"
   "b*(a*(b+a)+b)+a*(a+b*(b+a))"
*/

public class S1Parse_No_Error_Checking
{
   public static void recognizeS1(String stringInS1)
   {
      Tokenizer tokens = new Tokenizer(stringInS1);

      getS1(tokens);
   }//parecognizeS1()


   public static void getS1(Tokenizer tokens)
   {
      String token = tokens.peekToken();  // one token look ahead

      if ( token.equals("a") )            // decide which case we're in
      {
         getA(tokens);
         tokens.match("+");
         getB(tokens);
      }
      else
      {
         getB(tokens);
         tokens.match("+");
         getA(tokens);
      }
   }//getS1()


   public static void getA(Tokenizer tokens)
   {
      tokens.match("a");
      if ( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         tokens.nextToken();  // consume the *
         tokens.match("(");
         getS1(tokens);
         tokens.match(")");
      }
   }//getA()


   public static void getB(Tokenizer tokens)
   {
      tokens.match("b");
      if ( tokens.hasToken() && tokens.peekToken().equals("*") )
      {
         tokens.nextToken();  // consume the *
         tokens.match("(");
         getS1(tokens);
         tokens.match(")");
      }
   }//getB()





   public static void main(String[] args)
   {
      String[] stringsInS1 = {
                               "a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )"
                             };

      for (int i = 0; i < stringsInS1.length; i++)
      {
         System.out.println( "Parsing " + stringsInS1[i] );
         recognizeS1( stringsInS1[i] );
         System.out.println( " ... Parsed" );  // maybe not
      }
   }//main()

}//S1Parse_No_Error_Checking