/*
   Silly Language S1.

   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if the string cannot be parsed.

   Notice that these productions are mutually recursive, but no production is recursive.

   S3 -> A '+' B | B '+' A | A | B
   A  -> 'a' '*' '(' S3 ')' | 'a'
   B  -> 'b' '*' '(' S3 ')' | 'b'

   We can use EBNF to rewrite this grammar.

   S3 -> A [ '+' B ]  |  B [ '+' A ]
   A  -> 'a' [ '*' '(' S3 ')' ]
   B  -> 'b' [ '*' '(' S3 ')' ]

   Some example strings are:
   "a*(a+b)+b"
   "b*( a*(b+a)+b ) + a*(a+b*(b+a))"
   "a*(a)+b"
   "b*(a*(b+a))+a*(b*(b+a))"
*/

public class S3Parse
{
   public static void recognizeS3(String stringInS3) throws Exception
   {
      Tokenizer tokens = new Tokenizer(stringInS3);

      getS3(tokens);
   }//parecognizeS1()


   public static void getS3(Tokenizer tokens) throws Exception
   {
      String token = tokens.peekToken();  // we "peek" ahead to help us
      if ( token.equals("a") )            // decide which case we're in
      {
         getA(tokens);
         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();
            if ( token.equals("+") )
            {
               tokens.nextToken();  // consume the +
               getB(tokens);
            }
         }
      }
      else if ( token.equals("b") )
      {
         getB(tokens);
         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();
            if ( token.equals("+") )
            {
               tokens.nextToken();  // consume the +
               getA(tokens);
            }
         }
      }
      else
         throw new Exception("Can't parse.");
   }//getS1()


   public static void getA(Tokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("a") )
         throw new Exception("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new Exception("Can't parse " + token);
            getS3(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new Exception("Can't parse " + token);
         }
      }
   }//getA()


   public static void getB(Tokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("b") )
         throw new Exception("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new Exception("Can't parse " + token);
            getS3(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new Exception("Can't parse " + token);
         }
      }
   }//getB()





   public static void main(String[] args) throws Exception
   {
      String[] stringsInS3 = { "a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "a * ( a ) + b",
                               "b * ( a * ( b + a ) ) + a * ( b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )" };

      for (int i = 0; i < stringsInS3.length; i++)
      {
         System.out.println( "Parsing " + stringsInS3[i] );
         recognizeS3( stringsInS3[i] );
         System.out.println( " ... Parsed" );
      }
   }//main()

}//S3Parse